#!/usr/bin/env python

"""
:copyright: (c) 2013-2014 by Mike Taylor and Kartik Prabhu
:license: MIT, see LICENSE for more details.

IndieWeb Webmention Tools
"""

import os
import sys
import json
import re
from urlparse import urlsplit, urlunsplit

import requests


cfgFilenames = ('ronkyuu.cfg', '.ronkyuu.cfg')
cfgFilepaths = (os.getcwd(), '~/', '~/.ronkyuu/')

possibleConfigFiles = []

for p in cfgFilepaths:
    for f in cfgFilenames:
        possibleConfigFiles.append(os.path.join(p, f))


def discoverConfig(cfgFilename=None):
    result = {}

    if cfgFilename is None:
        for possibleFile in possibleConfigFiles:
            possibleFile = os.path.expanduser(possibleFile)
            if os.path.exists(possibleFile):
                result = json.load(open(possibleFile, 'r'))
                break
    else:
        possibleFile = os.path.expanduser(cfgFilename)
        if os.path.exists(possibleFile):
            result = json.load(open(possibleFile, 'r'))

    return result


def getURLChain(targetURL):
    ok = False
    chain = []
    try:
        r = requests.head(targetURL, allow_redirects=True)
        ok = r.status_code == requests.codes.ok
        if ok:
            for resp in r.history:
                chain.append(r.url)
    except:
        ok = False
    return (ok, chain)


def normalizeURL(targetURL):
    result = None
    ok, chain = getURLChain(targetURL)
    if ok:
        if len(chain) > 0:
            result = chain[-1]
        else:
            result = targetURL
    return result

def parse_link_header(link):
    """takes the link header as a string and returns a dictionary with rel values as keys and urls as values
    :param link: link header as a string
    :rtype: dictionary {rel_name: rel_value}
    """
    rel_dict = {}
    for rels in link.split(','):
        rel_break = rels.split(';')
        try:
            rel_url = re.search('<(.+?)>', rel_break[0]).group(1)
            rel_names = re.search('rel="(.+?)"', rel_break[1]).group(1)
            for name in rel_names.split():
                rel_dict[name] = rel_url
        except (AttributeError, IndexError):
            pass

    return rel_dict




#!/usr/bin/env python

"""
:copyright: (c) 2013 by Mike Taylor
:license: MIT, see LICENSE for more details.

IndieWeb Webmention Tools
"""

import argparse
import requests
import ronkyuu

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('sourceURL')
    parser.add_argument('--eventConfigFile', default=None)

    args = parser.parse_args()
    cfg  = ronkyuu.discoverConfig(args.eventConfigFile)

    domains   = [] #cfg.get('domains', [])
    sourceURL = args.sourceURL

    print('Scanning %s for mentions' % sourceURL)

    mentions = ronkyuu.findMentions(sourceURL, domains)
    print mentions['refs']
    for href in mentions['refs']:
        if sourceURL <> href:
            print href
            wmStatus, wmUrl = ronkyuu.discoverEndpoint(href, test_urls=False)
            if wmUrl is not None and wmStatus == 200:
                print('\tfound webmention endpoint %s for %s' % (wmUrl, href))
                status_code = ronkyuu.sendWebmention(sourceURL, href, wmUrl)

                if status_code == requests.codes.ok:
                    print('\twebmention sent successfully')
                else:
                    print('\twebmention send returned a status code of %s' % status_code)



#!/usr/bin/env python

"""
:copyright: (c) 2013-2014 by Mike Taylor and Kartik Prabhu
:license: MIT, see LICENSE for more details.

IndieWeb Webmention Tools
"""

import os
import sys

import requests
import re
from urlparse import urlparse, urljoin
from bs4 import BeautifulSoup, SoupStrainer

from validators import URLValidator
from .tools import parse_link_header


# User Aaron posts a blog post on his blog
# User Barnaby writes post on his blog that links to Aaron's post.
# After publishing the post (i.e. it has a URL), Barnaby's server notices this link as part of the publishing process
# Barnaby's server does webmention discovery on Aaron's post to find its webmention endpoint (if not found, process stops)
# Barnaby's server sends a webmention to Aaron's post's webmention endpoint with
#     source set to Barnaby's post's permalink
#     target set to Aaron's post's permalink.
# Aaron's server receives the webmention
# Aaron's server verifies that target (after following redirects) in the webmention is a valid permalink on Aaron's blog (if not, processing stops)
# Aaron's server verifies that the source (when retrieved, after following
# redirects) in the webmention contains a hyperlink to the target (if not,
# processing stops)


def findMentions(sourceURL, exclude_domains=[], content=None, look_in={'name':'body'}, test_urls=True):
    """Find all <a /> elements in the given html for a post. Only scan html element matching all criteria in look_in.

    optionally the content to be scanned can be given as an argument.
       
    If any have an href attribute that is not from the
    one of the items in exclude_domains, append it to our lists.

    :param sourceURL: the URL for the post we are scanning
    :param exclude_domains: a list of domains to exclude from the search
    :param content: the content to be scanned for mentions
    :param look_in: dictionary with name, id and class_. only element matching all of these will be scanned
    :param test_urls: optional flag to test URLs for validation
    :type exclude_domains: list
    :rtype: dictionary of Mentions
    """
    if test_urls:
        URLValidator(message='invalid source URL')(sourceURL)

    if content:
        result = {'status':   requests.codes.ok,
                  'headers':  None,
                  'content':  content
                  }
    else:
        r = requests.get(sourceURL, verify=False)
        result = {'status':   r.status_code,
                  'headers':  r.headers
                  }
        ## check for character encodings and use 'correct' data
        if 'charset' in r.headers.get('content-type', ''):
            result['content'] = r.text
        else:
            result['content'] = r.content

    result.update({'refs': set(), 'post-url': sourceURL})

    if result['status'] == requests.codes.ok:
        all_links = BeautifulSoup(result['content'], parse_only=SoupStrainer(**look_in)).find_all('a')
    
        for link in all_links:
            href = link.get('href', None)
            if href:
                url = urlparse(href)

                if url.scheme in ('http', 'https'):
                    if url.hostname and url.hostname not in exclude_domains:
                        result['refs'].add(href)
    return result


def findEndpoint(html):
    """Search the given html content for all <link /> elements
    and return any discovered WebMention URL.

    :param html: html content
    :rtype: WebMention URL
    """

    poss_rels = ['webmention', 'http://webmention.org', 'http://webmention.org/', 'https://webmention.org', 'https://webmention.org/']

    # find elements with correct rels and a href value
    all_links = BeautifulSoup(html).find_all(rel=poss_rels, href=True)
    for link in all_links:
        if link.get('href', ''):
            return link.get('href', '')

    return None


def discoverEndpoint(url, test_urls=True):
    """Discover any WebMention endpoint for a given URL.

    :param link: URL to discover WebMention endpoint
    :param test_urls: optional flag to test URLs for validation
    :rtype: tuple (status_code, URL)
    """
    if test_urls:
        URLValidator(message='invalid URL')(url)

    # status, webmention
    href = None
    r    = requests.get(url,verify=False)
    if r.status_code == requests.codes.ok:
        try:
            link = parse_link_header(r.headers['link'])
            href = link.get('webmention', '') or link.get('http://webmention.org', '') or link.get('http://webmention.org/', '') or link.get('https://webmention.org', '') or link.get('https://webmention.org/', '')

            # force searching in the HTML if not found
            if not href:
                raise AttributeError
        except (KeyError, AttributeError):
            href = findEndpoint(r.text)

        if href is not None:
            href = urljoin(url, href)

    return (r.status_code, href)


def sendWebmention(sourceURL, targetURL, webmention=None, test_urls=True):
    """Send to the :targetURL: a WebMention for the :sourceURL:

    The WebMention will be discovered if not given in the :webmention:
    parameter.

    :param sourceURL: URL that is referencing :targetURL:
    :param targetURL: URL of mentioned post
    :param webmention: optional WebMention endpoint
    :param test_urls: optional flag to test URLs for validation
    :rtype: HTTPrequest object if WebMention endpoint was valid
    """
    if test_urls:
        v = URLValidator()
        v(sourceURL)
        v(targetURL)

    result = None
    if webmention is None:
        wStatus, wUrl = discoverEndpoint(targetURL)
    else:
        wStatus = 200
        wUrl = webmention

    if wStatus == requests.codes.ok and wUrl is not None:
        if test_urls:
            v(wUrl)

        payload = {'source': sourceURL,
                   'target': targetURL}
        print 'sending to', wUrl, payload
        try:
            result = requests.post(wUrl, data=payload)
        except:
            result = None
    return result



#!/usr/bin/env python

"""
:copyright: (c) 2013-2014 by Mike Taylor and Kartik Prabhu
:license: MIT, see LICENSE for more details.

IndieWeb Rel=Me Tools
"""

import os
import sys

from tools import getURLChain, normalizeURL

import requests
from urlparse import urlparse
from bs4 import BeautifulSoup


# see http://microformats.org/wiki/rel-me for the spec
#
# With a profile URL http://bear.im and a resource URL http://twitter.com/bear
# is the resource an authoritative relme of the profile?
#
#   within the profile URL the following rel="me" links appear
#     <a href="http://twitter.com/bear" rel="me">@bear</a>
#     <a href="https://github.com/bear" rel="me" class="u-url">GitHub</a>
#
#   within the resource URL the following rel="me" links appear
#     <a target="_blank" rel="me nofollow" href="https://t.co/ZK4BFjSq66" class="js-tooltip" title="https://bear.im">bear.im</a>
#   and redirects
#     https://t.co/ZK4BFjSq66 -> [301] -> https://bear.im
#
#   The answer would be yes because the normalized resource relme link matches our profile URL
#
# With a profile URL http://dreev.es and a resource URL https://twitter.com/dreev
# is the resource an authoritative relme of the profile?
#
#   The profile URL itself redirects:
#       http://dreev.es -> [301] -> http://ai.eecs.umich.edu/people/dreeves/
#     and contains the following rel="me" links:
#       <a href="https://twitter.com/dreev" rel="me"/>
#
#   The resource URL has the following rel="me" links
#       <a target="_blank" rel="me nofollow" href="https://t.co/PlBCqLVndT" class="js-tooltip" title="http://dreev.es</a>
#     and redirects
#       http://t.co/PlBCqLVndT -> [301] -> http://dreev.es -> [301] -> http://ai.eecs.umich.edu/people/dreeves/
#
#   The answer would be yes because the normalized profile URL matches the normalized resource relme link
#
#   Other questions:
#     would it also be yes because the given profile URL matches one of the resource relme link's redirect chain urls?
#     do we then consider it as authorative if the chain segments match for the redirect?
#     i.e.
#                                                                              http://dreev.es -> [301] -> http://ai.eecs.umich.edu/people/dreeves/
#  https://twitter.com/dreev -> [rel-me] -> http://t.co/PlBCqLVndT -> [301] -> http://dreev.es -> [301] -> http://ai.eecs.umich.edu/people/dreeves/
#


def findRelMe(sourceURL):
    """Find all <a /> elements in the given html for a post.
       
    If any have an href attribute that is rel="me" then include
    it in the result.

    :param sourceURL: the URL for the post we are scanning
    :rtype: dictionary of RelMe references
    """
    r = requests.get(sourceURL)
    result = {'status':  r.status_code,
              'headers': r.headers,
              'history': r.history,
              'content': r.text,
              'relme':   [],
              'url':     sourceURL
              }
    if r.status_code == requests.codes.ok:
        dom = BeautifulSoup(r.text)

        for link in dom.find_all('a', rel='me'):
            rel  = link.get('rel')
            href = link.get('href')
            if rel is not None and href is not None:
                url = urlparse(href)
                if url is not None and url.scheme in ('http', 'https'):
                    result['relme'].append(href)
    return result


def confirmRelMe(profileURL, resourceURL, profileRelMes=None, resourceRelMes=None):
    """Determine if a given :resourceURL: is authoritative for the :profileURL:

    TODO add https/http filtering for those who wish to limit/restrict urls to match fully
    TODO add code to ensure that each item in the redirect chain is authoritative

    :param profileURL: URL of the user
    :param resourceURL: URL of the resource to validate
    :param profileRelMes: optional list of rel="me" links within the profile URL
    :param resourceRelMes: optional list of rel="me" links found within resource URL
    :rtype: True if confirmed
    """
    result  = False
    profile = normalizeURL(profileURL)

    if profileRelMes is None:
        profileRelMe = findRelMe(profileURL)
        profileRelMes = profileRelMe['relme']
    if resourceRelMes is None:
        resourceRelMe = findRelMe(resourceURL)
        resourceRelMes = resourceRelMe['relme']

    for url in resourceRelMes:
        if profile in (url, normalizeURL(url)):
            result = True
            break

    return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

# use requirements.txt for dependencies
with open('requirements.txt') as f:
    required = map(lambda s: s.strip(), f.readlines())

with open('README.md') as f:
    readme = f.read()

with open('LICENSE') as f:
    license = f.read()

setup(
    name='ronkyuu',
    version='0.2.7',
    description='Webmention Manager',
    long_description=readme,
    install_requires=required,
    author='Mike Taylor and Kartik Prabhu',
    author_email='bear@bear.im',
    url='https://github.com/bear/ronkyuu',
    license=license,
    packages=find_packages(exclude=('tests', 'docs'))
)


#!/usr/bin/env python

"""
:copyright: (c) 2013-2014 by Mike Taylor and Kartik Prabhu
:license: MIT, see LICENSE for more details.

IndieWeb URL Validation

Regex and URL validators borrowed from django.core.validators
https://github.com/django/django/blob/master/django/core/validators.py

    Copyright (c) Django Software Foundation and individual contributors.
    All rights reserved.
    BSD Licensed
"""

import re
from urlparse import urlsplit, urlunsplit

class RegexValidator(object):
    regex = ''
    message = 'Enter a valid value.'

    def __init__(self, regex=None, message=None):
        if regex is not None:
            self.regex = regex
        if message is not None:
            self.message = message

        # Compile the regex if it was not passed pre-compiled.
        if isinstance(self.regex, str):
            self.regex = re.compile(self.regex)

    def __call__(self, value):
        """
        Validates that the input matches the regular expression.
        """
        if not self.regex.search(value):
            raise ValueError(self.message)

    def __eq__(self, other):
        return isinstance(other, RegexValidator) and (self.regex == other.regex) and (self.message == other.message)


class URLValidator(RegexValidator):
    regex = re.compile(
        r'^(?:[a-z0-9\.\-]*)://'  # scheme is validated separately
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
        r'localhost|'  # localhost...
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}|'  # ...or ipv4
        r'\[?[A-F0-9]*:[A-F0-9:]+\]?)'  # ...or ipv6
        r'(?::\d+)?'  # optional port
        r'(?:/?|[/?]\S+)$', re.IGNORECASE)
    message = 'Enter a valid URL.'
    # check only http and https
    schemes = ['http', 'https']

    def __init__(self, schemes=None, **kwargs):
        super(URLValidator, self).__init__(**kwargs)
        if schemes is not None:
            self.schemes = schemes

    def __call__(self, value):
        e = ValueError(self.message)
        #check for None, and empty string
        if value in (None, ''):
            raise e
        value = str(value)
        # Check first if the scheme is valid
        scheme = value.split('://')[0].lower()
        if scheme not in self.schemes:
            raise e

        # Then check full URL
        try:
            super(URLValidator, self).__call__(value)
        except ValueError:
            # Trivial case failed. Try for possible IDN domain
            if value:
                scheme, netloc, path, query, fragment = urlsplit(value)
                try:
                    netloc = netloc.encode('idna').decode('ascii')  # IDN -> ACE
                except UnicodeError:  # invalid domain part
                    raise e
                url = urlunsplit((scheme, netloc, path, query, fragment))
                super(URLValidator, self).__call__(url)
            else:
                raise e
        else:
            url = value
